#include <OGRE/Ogre.h>

#include <exception>
#include <stdexcept>

int main(int argc, char** argv)
{

  Ogre::Root* root = new Ogre::Root();
  root->loadPlugin( "RenderSystem_GL" );
  root->loadPlugin( "Plugin_CgProgramManager" );
  root->loadPlugin( "Plugin_OctreeSceneManager" );
  root->loadPlugin( "Plugin_ParticleFX" );
  
  if (!root->showConfigDialog())
    {
      return 1;
    }

  Ogre::RenderWindow* window = root->initialise(true);
  Ogre::SceneManager* scene_manager = root->createSceneManager( Ogre::ST_GENERIC, "TestSceneManager" );
  Ogre::Camera* camera = scene_manager->createCamera("Cam");
  camera->setNearClipDistance(0.1);
  camera->setPosition(Ogre::Vector3(0, 0, 5));
  camera->lookAt(Ogre::Vector3(0, 0, 0));

  // Create one viewport, entire window
  Ogre::Viewport* vp = window->addViewport(camera);
  vp->setBackgroundColour(Ogre::ColourValue(1,0,0));
  // Alter the camera aspect ratio to match the viewport
  camera->setAspectRatio(Ogre::Real(vp->getActualWidth()) / Ogre::Real(vp->getActualHeight()));
  
  Ogre::ResourceGroupManager::getSingleton().addResourceLocation("media", "FileSystem", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
  Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

  Ogre::Overlay* overlay = Ogre::OverlayManager::getSingleton().create("test");
  Ogre::OverlayContainer* container = (Ogre::OverlayContainer*)Ogre::OverlayManager::getSingleton().createOverlayElement("Panel", "container1");
  overlay->add2D(container);
  overlay->show();

  Ogre::Entity* ent = scene_manager->createEntity("entity", "sphere.mesh");
  scene_manager->getRootSceneNode()->createChildSceneNode()->attachObject(ent);
  
  Ogre::OverlayElement* text = 0;
  
  int i = 0;

  while (true)
    {
      if (i % 1000 == 0)
	{
	  if (!text)
	    {
	      printf("Creating text\n");
	      text = Ogre::OverlayManager::getSingleton().createOverlayElement("TextArea", "text");
	      text->setParameter("font_name", "Ogre");
	      text->setParameter("char_height", "1.0");
	      text->setDimensions(100.0, 100.0);
	      text->setColour(Ogre::ColourValue(1.0, 1.0, 1.0, 1.0));
	      text->setCaption("A");
	      container->addChild(text);
	    }
	  else
	    {
	      printf("Deleting text\n");
	      container->removeChild(text->getName());
	      Ogre::OverlayManager::getSingleton().destroyOverlayElement("text");
	      text = 0;
	    }
	}
      
      root->renderOneFrame();

      ++i;
    }
  
  delete root;
}

